@@ -24,6 +24,7 @@ import timeout_decorator
 import yaml
 from ansi2html import Ansi2HTMLConverter
 from dateutil import parser
+from docker import DockerClient
 from docker.models.containers import Container
 from duckietown_challenges import __version__ as version_dc, ChallengesConstants, get_duckietown_server_url
 from duckietown_challenges.challenge import ChallengeDescription, EvaluationParameters
@@ -57,6 +58,7 @@ from zuper_commons.fs import write_ustring_to_utf8_file
 from zuper_commons.text import box
 from zuper_commons.types import ZException
 
+
 from . import __version__, __version__ as version_dcr
 from .env_checks import (
     check_docker_environment,
@@ -71,6 +73,10 @@ from .shell_config import DTShellConstants, read_shell_config
 from .types import ContainerName
 
 
+def is_solution_container(container_name: str) -> bool:
+    return "solution" in container_name
+
+
 def get_token_from_shell_config():
     k = DTShellConstants.DT1_TOKEN_CONFIG_KEY
 
@@ -495,7 +501,7 @@ def go_(
         msg = "Could not find jobs."
         raise NothingLeft(msg, res=res)
     job_id = res["job_id"]
-    elogger.info(res=res)
+    # elogger.info(res=res)
 
     if False:
         pass
@@ -766,6 +772,7 @@ def run_single(
     project: str,
     do_pull: bool,
     server_host: Optional[str],
+    quiet: bool = False,
     debug_volumes: Optional[str] = None,
     timeout_sec: float,
     require_scores: Set[str],
@@ -794,6 +801,7 @@ def run_single(
     :param allow_host_network:
     :param use_ipfs:
     :param heartbeat:
+    :param quiet: minimal output
 
     :return:
     """
@@ -809,17 +817,12 @@ def run_single(
     if not tomonitor:
         elogger.error(f"Cannot find any container to monitor among {list(challenge_parameters.services)}")
 
-    elogger.info(f"The containers to monitor: {tomonitor}")
+    elogger.debug(f"The containers to monitor: {tomonitor}")
     add_extra_environment = {}
     add_extra_environment[ENV_CHALLENGE_NAME] = challenge_name
     add_extra_environment[ENV_CHALLENGE_STEP_NAME] = challenge_step_name
     # add_extra_environment["submission_id"] = submission_id
     add_extra_environment[ChallengesConstants.SUBMISSION_CONTAINER_TAG] = solution_container
-    #
-    # fd = os.path.join(wd, "fifos")
-    # if os.path.exists(fd):
-    #     shutil.rmtree(fd)
-    # os.makedirs(fd)
 
     config = get_config(
         wd=wd,
@@ -831,6 +834,7 @@ def run_single(
         allow_host_network=allow_host_network,
         add_extra_environment=add_extra_environment,
         use_ipfs=use_ipfs,
+        quiet=quiet,
     )
     config_yaml = yaml.safe_dump(config, encoding="utf-8", indent=4, allow_unicode=True)
     if isinstance(config_yaml, bytes):
@@ -849,14 +853,23 @@ def run_single(
     try:
         config_validated = run_docker(wd, project, ["config"], get_output=True).decode("utf-8")
         write_ustring_to_utf8_file(config_validated, dcfn)
-        elogger.info(config_orig=config_yaml, config_validated=config_validated)
+        # elogger.info(config_orig=config_yaml, config_validated=config_validated)
         valid_config = True
         valid_config_error = None
     except DockerComposeFail:
         valid_config_error = "Could not validate Docker Compose configuration:\n%s" % traceback.format_exc()
         elogger.error(valid_config_error)
         valid_config = False
-
+    #
+    #
+    # if do_pull:
+    #     from duckietown_challenges_cli.cli_pull import docker_pull_retry
+    #     client = DockerClient.from_env()
+    #     for service_name, s in config["services"].items():
+    #         image = s["image"]
+    #         if "dummy" in image:
+    #             continue
+    #         docker_pull_retry(client, image, ntimes=4, wait=5)
     if valid_config:
         services_names = list(challenge_parameters.services)
         # print('services: %s' % services_names)
@@ -869,6 +882,7 @@ def run_single(
             timeout_sec=timeout_sec,
             heartbeat=heartbeat,
             quota_cpu=quota_cpu,
+            quiet=quiet,
         )
 
         write_logs(wd, project, services=config["services"])
@@ -895,18 +909,52 @@ def get_id_for_service(wd, project, service_name) -> str:
     try:
         o = run_docker(wd, project, cmd, get_output=True)
         container_id = o.decode("utf-8").strip()  # \n at the end
+        if not container_id:
+            msg = f"Cannot get ID for servicee {service_name!r}."
+            raise DockerComposeFail(msg, o=o)
     except DockerComposeFail:
         raise
     return container_id
 
 
+from zuper_commons.types import ZValueError
+
+
+def get_services_id(wd, project, services: List[str]) -> Dict[str, str]:
+    cmd = ["ps", "-q"]
+    cmd.extend(services)
+
+    try:
+        o = run_docker(wd, project, cmd, get_output=True)
+    except DockerComposeFail:
+        raise
+    container_ids = o.decode("utf-8").strip().split("\n")  # \n at the end
+    # Note: unfortunately the container_ids are not ordered
+    res = {}
+    client = DockerClient.from_env()
+    names = {}
+    for container_id in container_ids:
+        container = client.containers.get(container_id)
+        names[container_id] = container.name
+        for s in services:
+            if s in container.name:
+                res[s] = container_id
+    if len(res) != len(services):
+        raise ZValueError(container_ids=container_ids, services=services, res=res, names=names)
+
+    return res
+
+
 class EvaluatorTimeout(Exception):
     pass
 
 
-def write_logs_worker(service_name, container_id):
-    # elogger.info('writing logs for %s ' % container_id)
+colors = ["red", "blue", "green", "yellow"]
 
+
+def write_logs_worker(service_name: str, container_id: str, color: str, l: int):
+    # elogger.info('writing logs for %s ' % container_id)
+    scol = termcolor.colored(service_name.ljust(l), color)
     client = check_docker_environment()
 
     container = client.containers.get(container_id)
@@ -916,7 +964,8 @@ def write_logs_worker(service_name, container_id):
         timestamp = x[:i]
         message = x[i + 1 :].rstrip()  # \n
         ts = parser.parse(timestamp)
-        s = f"{ts.strftime('%H:%M:%S')} {service_name} {message}"
+
+        s = f"{ts.strftime('%H:%M:%S')} {scol} {message}"
         sys.stdout.write(s + "\n")
         sys.stdout.flush()
         if container.status != "running":
@@ -924,7 +973,7 @@ def write_logs_worker(service_name, container_id):
 
     exit_code = exit_code_for_container(container_id)
 
-    elogger.info(f"container {service_name} terminated with {exit_code}")
+    elogger.debug(f"write_logs_worker: container {service_name} terminated with {exit_code}")
 
 
 def exit_code_for_container(container_id):
@@ -937,10 +986,10 @@ class Workers(object):
     workers = []
 
 
-def stream_logs(service_name, container_id):
+def stream_logs(service_name: str, container_id: str, color: str, l: int):
     from multiprocessing import Process
 
-    p = Process(target=write_logs_worker, args=(service_name, container_id))
+    p = Process(target=write_logs_worker, args=(service_name, container_id, color, l))
     p.start()
     Workers.workers.append(p)
 
@@ -959,10 +1008,11 @@ def run_one(
     project: str,
     do_pull: bool,
     monitor: ContainerName,
-    services,
+    services: List[str],
     timeout_sec: float,
     quota_cpu: Optional[float],
     heartbeat: Optional[Callable] = None,
+    quiet: bool = False,
 ):
     """
 
@@ -974,16 +1024,17 @@ def run_one(
     :param timeout_sec:
     :param quota_cpu: if it is not None, this is the avg number of cpus
     :param heartbeat:
+    :param quiet: minimal output
     :return:
     """
     client = check_docker_environment()
 
     # noinspection PyBroadException
     try:
-        if do_pull:
-            elogger.info("pulling containers...")
-            cmd = ["pull"]  # '--ignore-pull-failures'
-            run_docker(wd, project, cmd)
+        # if do_pull:
+        #     elogger.info("pulling containers...")
+        #     cmd = ["pull"]  # '--ignore-pull-failures'
+        #     run_docker(wd, project, cmd)
 
         try:
             _pruned = client.networks.prune(filters=dict(until="1h"))
@@ -1001,24 +1052,20 @@ def run_one(
             # '--abort-on-container-exit'
         ]
         run_docker(wd, project, cmd)
-        elogger.info("Running containers %s" % services)
+        elogger.debug(f"Creating containers {services}")
         cmd = [
             "up",
-            "-d",
+            "--no-start",
             "--renew-anon-volumes",
-            # '--remove-orphans',
-            # '--abort-on-container-exit'
         ]
-        elogger.info("now running", wd=wd, cwd=os.getcwd(), wd_files=list_all_files(wd))
         run_docker(wd, project, cmd)
 
-        service2id = {}
-        for service_name in services:
-            service2id[service_name] = get_id_for_service(wd, project, service_name)
-            stream_logs(service_name, service2id[service_name])
         t0 = time.time()
 
         client = check_docker_environment()
+        service2id = get_services_id(wd, project, services)
+        # for service_name in services:
+        #     service2id[service_name] = get_id_for_service(wd, project, service_name)
 
         for service_name, container_id in service2id.items():
             container: Container = client.containers.get(container_id)
@@ -1030,7 +1077,9 @@ def run_one(
 
             container.update(cpu_shares=250)
 
-            if "solution" in service_name:
+            if is_solution_container(service_name):
+                M = "8g"
+                container.update(memswap_limit=-1, mem_reservation=M, mem_limit=M)
                 if quota_cpu is not None:
                     # every second
                     cpu_period = 1000 * 1000
@@ -1048,7 +1097,7 @@ def run_one(
             # noinspection PyUnresolvedReferences
             ports = container.ports
             # {'10123/tcp': [{'HostIp': '0.0.0.0', 'HostPort': '32769'}]}
-            elogger.info(f"container ports: {ports}")
+            # elogger.debug(f"container ports: {ports}")
             for port_proto, forwards in ports.items():
                 for forward in forwards:
                     if "HostPort" in forward:
@@ -1059,20 +1108,44 @@ def run_one(
                         msg = "\n" * 10 + box(msg) + "\n" * 2
                         elogger.info(msg)
 
-        exited = []
+        elogger.debug(f"Starting containers {services}")
+        cmd = [
+            "up",
+            "-d",
+        ]
+        run_docker(wd, project, cmd)
+
+        l = max(len(_) for _ in service2id)
+        for i, (service_name, service_id) in enumerate(service2id.items()):
+            if quiet and (not is_solution_container(service_name)):
+                continue
+            color = colors[i % len(colors)]
+            stream_logs(service_name, service_id, color=color, l=l)
+        exited2 = {}
         while True:
             if heartbeat:
                 heartbeat()
             running = []
+            running_details = {}
             for service_name, container_id in service2id.items():
-                if service_name in exited:
+                if service_name in exited2:
                     continue
                 container = client.containers.get(container_id)
+                stats = container.stats(stream=False)
+                try:
+                    memory_stats1 = int(stats["memory_stats"]["usage"] / (1024 * 1024.0))
+                    memory_stats = f"{memory_stats1} MB RAM"
+                except KeyError:
+                    memory_stats = "n/a"
+                cpu_stats = stats["cpu_stats"]["cpu_usage"]["total_usage"] / (1024 * 1024.0)
+                running_details[service_name] = f"{memory_stats}; cpu {cpu_stats}"
+                # elogger.debug(container_id=container_id, stats=stats)
                 # msg = 'service %s: %s' % (service_name, container.status)
                 # elogger.debug(msg)
                 if container.status != "running":
-                    exited.append(service_name)
+
                     exit_code = exit_code_for_container(container_id)
+                    exited2[service_name] = exit_code
                     if exit_code:
                         msg = f'The container "{service_name}" exited with code {exit_code}.\n'
                         if exit_code == 137:
@@ -1081,9 +1154,18 @@ def run_one(
                                 "give "
                                 "Docker enough RAM."
                             )
+                            status = ChallengesConstants.STATUS_JOB_HOST_ERROR
+                            cr = ChallengeResults(status, msg, scores={})
+                            return cr
 
                         msg += "\n\nLook at the logs for the container to know more about the error."
                         elogger.error(msg)
+
+                        if is_solution_container(service_name):
+                            status = ChallengesConstants.STATUS_JOB_FAILED
+                            cr = ChallengeResults(status, msg, scores={})
+                            return cr
+
                         try:
                             cr = read_challenge_results(wd)
                         except NoResultsFound:
@@ -1097,18 +1179,15 @@ def run_one(
                     running.append(service_name)
 
             delta = time.time() - t0
-            msg = (
-                f"Running for {int(delta)} s (timeout: {int(timeout_sec)} s). Services running: {running}  "
-                f"exited:  {exited}"
-            )
-            elogger.debug(msg)
+            msg = f"Running for {int(delta)} s (timeout: {int(timeout_sec)} s). Services running: {running}  "
+            elogger.debug(msg, running_details=running_details, exited=exited2)
 
-            if monitor in exited:
-                elogger.info(f"The container {monitor!r} has exited; bye bye.")
+            if monitor in exited2:
+                elogger.info(f"The container to monitor {monitor!r} has exited; bye bye.", exited=exited2)
                 break
 
             if delta > timeout_sec:
-                msg = "Waited %d seconds for container to finish. Giving up. " % delta
+                msg = f"Waited {int(delta)} seconds for container to finish. Giving up. "
                 elogger.error(msg)
                 status = ChallengesConstants.STATUS_JOB_ERROR
                 cr = ChallengeResults(status, msg, scores={})
@@ -1194,6 +1273,7 @@ def get_config(
     debug_volumes: Optional[str] = None,
     allow_host_network: bool = False,
     use_ipfs: bool = False,
+    quiet: bool = False,
 ):
     """
         server_host: reachable from other ips
@@ -1216,38 +1296,15 @@ def get_config(
 
     config = challenge_parameters_.as_dict()
 
-    # do_trick = False
-    for service in config["services"].values():
+    for service_name, service in config["services"].items():
         image = service["image"]
         br = parse_complete_tag(image)
-        # br.tag = None
+        br.tag = None
         service["image"] = get_complete_tag(br)
-        #
-        # if do_trick:
-        #     if image_digest:
-        #         # If we have the image then we don't bother with pulling the tag
-        #         try:
-        #             c = client.images.get(image_digest)
-        #         except NotFound:
-        #             msg = 'Could not find the image for %s by digest %s; will need to pull.' % (
-        #                 service['image'], image_digest)
-        #             elogger.info(msg)
-        #         else:
-        #             msg = 'I already have the exact image for %s' % service['image']
-        #             elogger.info(msg)
-        #             #
-        #             #
-        #             if True:
-        #
-        #                 n = len('91bac885982d')
-        #                 partial_id = image_digest.replace('sha256:', '')[:n]
-        #                 service['image'] = partial_id
-        #             else:
-        #                 image = service['image']
-        #                 # remove tag
-        #                 if ':' in image:
-        #                     image = image.split(':')[0]
-        #                 service['image'] = image + '@' + image_digest
+
+        if quiet:
+            if not is_solution_container(service_name):
+                service["logging"] = {"driver": "none"}
 
         service.pop("build", None)
 
@@ -1257,11 +1314,6 @@ def get_config(
         # if image_digest is not None:
         #     service['image'] += '@' + image_digest
 
-    # else:
-    #     msg = 'Cannot find the tag %s' % SUBMISSION_CONTAINER_TAG
-    #     elogger.warning(msg)
-    #     # raise Exception(msg)
-
     def naturalize(image: str) -> str:
         if "localhost" in image:
             image2 = image.replace("localhost", server_host)
@@ -1290,6 +1342,8 @@ def get_config(
     #         }
     volumes0 = {}
     volumes0[wd] = {"bind": "/challenges", "mode": "rw"}
+    write_ustring_to_utf8_file("runner", os.path.join(fd, "runner"))
+    write_ustring_to_utf8_file("runner", os.path.join(wd, "runner"))
     volumes0[fd] = {"bind": "/fifos", "mode": "rw"}
     volumes0[dir_fake_home_host] = {"bind": dir_home_guest, "mode": "rw"}
 
@@ -1300,10 +1354,9 @@ def get_config(
             msg = "/ipfs mount point not found"
             raise IPFSException(msg)
 
-    if debug_volumes:
+    if debug_volumes or ("DT_MOUNT" in os.environ):
+        elogger.debug("Mounting local sources.")
         volumes0.update(get_developer_volumes())
-        # data = yaml.load(open(debug_volumes).read(), Loader=yaml.SafeLoader)
-        # volumes.extend(data["volumes"])
 
     for service in config["services"].values():
         assert "volumes" not in service
@@ -1325,26 +1378,20 @@ def get_config(
     return config
 
 
-def write_logs(wd, project, services):
+def write_logs(wd, project, services: List[str]):
     client = check_docker_environment()
 
     logdir = os.path.join(wd, "logs")
     if not os.path.exists(logdir):
         os.makedirs(logdir)
 
+    services2id: Dict[str, str] = get_services_id(wd, project, services)
     for service in services:
         try:
-            container_id = get_id_for_service(wd, project, service)
-
-            # cmd = ['ps', '-q', service]
-            # try:
-            #     o = run_docker(wd, project, cmd, get_output=True)
-            #     container_id = o.strip()  # \n at the end
-            # except DockerComposeFail:
-            #     continue
+            container_id = services2id[service]
 
             if not container_id:
-                logs = 'Service "%s" was not started.' % service
+                logs = f'Service "{service}" was not started.'
                 elogger.warning(logs)
             else:
                 # elogger.info('Found container ID = %r' % container_id)
@@ -1365,10 +1412,10 @@ def write_logs(wd, project, services):
                     logdir_service = os.path.join(logdir, service)
                     if not os.path.exists(logdir_service):
                         os.makedirs(logdir_service)
-                    fn = os.path.join(logdir_service, "%s.log" % fname)
+                    fn = os.path.join(logdir_service, f"{fname}.log")
                     write_ustring_to_utf8_file(logs, fn)
 
-                    fn = os.path.join(logdir_service, "%s.html" % fname)
+                    fn = os.path.join(logdir_service, f"{fname}.html")
                     write_ustring_to_utf8_file(html, fn)
         except:
             elogger.error(traceback.format_exc())
@@ -1579,12 +1626,14 @@ def download_artefacts(aws_config, steps2artefacts, wd, use_ipfs: bool):
                     raise CouldNotDownloadAll(msg)
 
 
+import boto3
+
+
 def try_s3(aws_config):
     bucket_name = aws_config["bucket_name"]
     aws_access_key_id = aws_config["aws_access_key_id"]
     aws_secret_access_key = aws_config["aws_secret_access_key"]
     aws_root_path = aws_config["path"]
-    import boto3
 
     s3 = boto3.resource(
         "s3", aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,
@@ -1601,7 +1650,6 @@ def try_s3(aws_config):
 def get_object(aws_config, bucket_name, object_key, fn):
     aws_access_key_id = aws_config["aws_access_key_id"]
     aws_secret_access_key = aws_config["aws_secret_access_key"]
-    import boto3
 
     s3 = boto3.resource(
         "s3", aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key,
@@ -1772,16 +1820,16 @@ def create_index_files(wd, job_id):
 def create_index(root, dirnames, filenames, job_id):
     s = "<html><head></head><body>\n"
 
-    url = DEFAULT_DTSERVER + "/humans/jobs/%s" % job_id
-    s += '<p>These are the output for <a href="%s">Job %s</a>' % (url, job_id)
+    url = DEFAULT_DTSERVER + f"/humans/jobs/{job_id}"
+    s += f'<p>These are the output for <a href="{url}">Job {job_id}</a>'
     s += "<table>"
 
     for d in dirnames:
-        s += '\n<tr><td></td><td><a href="%s">%s/</td></tr>' % (d, d)
+        s += f'\n<tr><td></td><td><a href="{d}">{d}/</td></tr>'
 
     for f in filenames:
         size = os.stat(os.path.join(root, f)).st_size
-        s += '\n<tr><td>%.3f MB</td><td><a href="%s">%s</td></tr>' % (size / (1024 * 1024.0), f, f,)
+        s += f'\n<tr><td>{size / (1024 * 1024.0):.3f} MB</td><td><a href="{f}">{f}</td></tr>'
 
     s += "\n</table>"
     s += "\n</body></head>"
