@@ -38,5 +38,5 @@ def copy_to_cache(fn: str, sha256hex: str):
     have = os.path.join(cache_dir_by_value, sha256hex)
     if not os.path.exists(have):
         msg = "Copying %s to cache %s" % (friendly_size(os.stat(fn).st_size), have)
-        logger.debug(msg)
+        # logger.debug(msg)
         shutil.copy(fn, have)
