@@ -1,5 +1,6 @@
 # coding=utf-8
 import argparse
+import logging
 import os
 import random
 import shutil
@@ -9,14 +10,14 @@ from datetime import datetime
 
 import termcolor
 import yaml
-from zuper_commons.logs import ZLogger
-
 from duckietown_challenges import CHALLENGE_PREVIOUS_STEPS_DIR, ChallengeResults, List, traceback
 from duckietown_challenges.rest_methods import dtserver_get_compatible_challenges, get_challenge_description
 from duckietown_challenges.submission_read import read_submission_info
 from duckietown_challenges.utils import indent
+
 from .env_checks import check_docker_environment
 from .exceptions import UserError
+from .logging import logger
 from .runner import get_token_from_shell_config, run_single
 from .submission_build import build_image
 
@@ -26,7 +27,7 @@ usage = """
 
 
 """
-logger = ZLogger("runner-local")
+# logger = ZLogger("runner-local")
 
 
 # noinspection PyBroadException
@@ -52,6 +53,8 @@ def runner_local_main_(args: List[str] = None, token: str = None):
 
         setup_logging_color()
     prog = "dts challenges evaluate"
+
+    # logger.info(env=dict(os.environ))
     parser = argparse.ArgumentParser(prog=prog, usage=usage)
 
     group = parser.add_argument_group("Basic")
@@ -59,6 +62,7 @@ def runner_local_main_(args: List[str] = None, token: str = None):
     group.add_argument("--no-cache", action="store_true", default=False, help="")
 
     group.add_argument("--no-build", action="store_true", default=False, help="")
+    group.add_argument("--no-pull", action="store_true", default=False, help="")
     group.add_argument("--dev", action="store_true", default=False, help="Activate development mode")
     group.add_argument("--output", default=None)
     group.add_argument("--impersonate", default=None)
@@ -67,15 +71,21 @@ def runner_local_main_(args: List[str] = None, token: str = None):
     parser.add_argument("--debug-volumes", default=None)
 
     parser.add_argument("--quota-cpu", type=float, default=None, help="average number of CPUs")
+    parser.add_argument(
+        "--quiet", default=False, action="store_true", help="Quiet output (only solution container)"
+    )
 
     group.add_argument("-C", dest="change", default=None)
 
     parsed = parser.parse_args(args=args)
     quota_cpu = parsed.quota_cpu
+    quiet = parsed.quiet
 
-    logger.debug("Running in directory %s" % os.getcwd())
+    if quiet:
+        logger.setLevel(logging.INFO)
 
     if parsed.change:
+        logger.debug("Running in directory %s" % os.getcwd())
         os.chdir(parsed.change)
         logger.debug("Changing to directory %s" % os.getcwd())
 
@@ -86,7 +96,7 @@ def runner_local_main_(args: List[str] = None, token: str = None):
     sub_info = read_submission_info(path)
 
     compat = dtserver_get_compatible_challenges(
-        token=token, impersonate=parsed.impersonate, submission_protocols=sub_info.protocols,
+        token=token, impersonate=parsed.impersonate, submission_protocols=sub_info.protocols, quiet=quiet,
     )
     if not compat.compatible:
         msg = "There are no compatible challenges with protocols %s." % sub_info.protocols
@@ -94,14 +104,14 @@ def runner_local_main_(args: List[str] = None, token: str = None):
 
     dockerfile = os.path.join(path, "Dockerfile")
     if not os.path.exists(dockerfile):
-        msg = 'I expected to find the file "%s".' % dockerfile
+        msg = f'I expected to find the file "{dockerfile}".'
         raise Exception(msg)
 
     client = check_docker_environment()
 
     no_cache = parsed.no_cache
     no_build = parsed.no_build
-    do_pull = False
+    do_pull = not parsed.no_pull
 
     if parsed.challenge is not None:
         sub_info.challenge_names = parsed.challenge.split(",")
@@ -110,7 +120,7 @@ def runner_local_main_(args: List[str] = None, token: str = None):
 
         for c in sub_info.challenge_names:
             if c not in compat.available_submit:
-                msg = 'The challenge "%s" is not available.' % c
+                msg = f'The challenge "{c}" is not available.'
                 # msg += '\n available %s' % list(compat.available_submit)
                 raise UserError(msg)
 
@@ -144,7 +154,7 @@ For example, you could try:
 
     one = sub_info.challenge_names[0]
 
-    logger.info("I will evaluate challenge %s" % one)
+    logger.debug(f"I will evaluate challenge {one}")
     cd = get_challenge_description(token, one, impersonate=parsed.impersonate)
     # logger.info(cd=cd)
 
@@ -157,17 +167,17 @@ For example, you could try:
     solution_container = tag
     SUCCESS = "success"
     steps_ordered = list(sorted(cd.steps))
-    logger.info("steps: %s" % steps_ordered)
+    logger.debug("steps", steps_ordered=steps_ordered)
     output = parsed.output
     if output is None:
         timestamp = datetime.now().strftime("%y_%m_%d_%H_%M_%S")
         output = os.path.join("/tmp", one, timestamp)
     logger.info(f"Using output directory {output}")
     for i, challenge_step_name in enumerate(steps_ordered):
-        logger.info('Now considering step "%s"' % challenge_step_name)
+        logger.debug(f'Now considering step "{challenge_step_name}"')
         step = cd.steps[challenge_step_name]
         evaluation_parameters_str = (
-            yaml.safe_dump(step.evaluation_parameters.as_dict()) + "\ns: %s" % solution_container
+            yaml.safe_dump(step.evaluation_parameters.as_dict()) + f"\ns: {solution_container}"
         )
 
         wd_final = os.path.join(output, challenge_step_name)
@@ -178,17 +188,14 @@ For example, you could try:
                     cr_yaml = open(os.path.join(wd_final, "results.yaml"))
                     cr = ChallengeResults.from_yaml(yaml.load(cr_yaml))
                     if cr.status == SUCCESS:
-                        msg = 'Not redoing step "%s" because it is already completed.' % challenge_step_name
+                        msg = f'Not redoing step "{challenge_step_name}" because it is already completed.'
                         logger.info(msg)
-                        msg = "If you want to re-do it, erase the directory %s." % wd_final
+                        msg = f"If you want to re-do it, erase the directory {wd_final}."
                         logger.info(msg)
                         continue
                     else:
-                        msg = 'Breaking because step "%s" was already evaluated with result "%s".' % (
-                            challenge_step_name,
-                            cr.status,
-                        )
-                        msg += "\n" + "If you want to re-do it, erase the directory %s." % wd_final
+                        msg = f'Breaking because step "{challenge_step_name}" was already evaluated with result "{cr.status}".'
+                        msg += "\n" + f"If you want to re-do it, erase the directory {wd_final}."
                         logger.error(msg)
                         break
                 else:
@@ -238,7 +245,7 @@ For example, you could try:
         server_host = None
         if "CONTAINER_PREFIX" in os.environ:
             prefix = os.environ["CONTAINER_PREFIX"]
-            logger.info(f"will respect CONTAINER_PREFIX = {prefix!r} ")
+            # logger.debug(f"will respect CONTAINER_PREFIX = {prefix!r} ")
         else:
             prefix = "noprefix"
             logger.warn(f"did not find CONTAINER_PREFIX; using {prefix!r} ")
@@ -264,6 +271,7 @@ For example, you could try:
             server_host=server_host,
             require_scores=require_scores,
             quota_cpu=quota_cpu,
+            quiet=quiet,
         )
 
         fn = os.path.join(wd, "results.yaml")
