@@ -1,8 +1,9 @@
 # coding=utf-8
 import subprocess
 
-from duckietown_challenges import get_important_env_build_args
+from zuper_commons.fs import read_ustring_from_utf8_file
 
+from duckietown_challenges import get_important_env_build_args
 from . import logger
 
 
@@ -11,14 +12,24 @@ def build_image(client, path: str, tag: str, dockerfile: str, no_build: bool, no
         cmd = ["docker", "build", "--pull", "-t", tag, "-f", dockerfile]
         if no_cache:
             cmd.append("--no-cache")
-        with open(dockerfile) as _:
-            df_contents = _.read()
+
+        df_contents = read_ustring_from_utf8_file(dockerfile)
 
         cmd.extend(get_important_env_build_args(df_contents))
 
         cmd.append(path)
 
-        logger.info("running command", cmd=cmd)
+        cmds = " ".join(cmd)
+        m = f"""
+
+        Running command:
+
+        $ {cmds}
+
+
+        """
+
+        logger.debug(m)
         try:
             subprocess.check_call(cmd)
         except subprocess.CalledProcessError as e:
